# Preside Extension: Basic REST Security

This is an extension for [Preside](https://www.presidecms.com) that provides a simple authentication mechanism for the in-built REST API architecture.

## Configuration

The extension provides a system configuration screen that allows you to configure:

* *API Key:* an API key that is required for each request to the PresideCMS REST API
* *HTTP Header name:* the HTTP request header that is checked for the provided API Key (defaults to X-API-KEY)

*if no API key is defined the system automatically generates one initially*.

## Usage

Use the configured API key in all REST request in a custom HTTP header. 

### cURL example

In the following example the header *X-API-KEY* is used and the configured valid API key is *3635F2A8CB840264EA3BC0FF179E25B8*

    curl -X GET --header 'Accept: application/json' --header 'X-API-KEY: 3635F2A8CB840264EA3BC0FF179E25B8' 'http://servername:serverport/api/myrestapiendpoint/'

### Results

The following responses are returned in case of errors. In these cases the response content is always empty and the detail error message is returned in the custom HTTP response header *X-ERROR-MESSAGE*.

    HTTP/1.1 401 Not Authenticated
    Connection: close
    X-ERROR-MESSAGE: Missing HTTP header 'X-API-KEY'
    Content-Type: application/json;charset=utf-8
    Content-Length: 0
    Date: Mon, 22 Feb 2016 14:37:16 GMT

    HTTP/1.1 403 Not Authorized
    Connection: close
    X-ERROR-MESSAGE: Invalid API Key. Please use a valid API key in HTTP header 'X-API-KEY'
    Content-Type: application/json;charset=utf-8
    Content-Length: 0
    Date: Mon, 22 Feb 2016 14:49:34 GMT


*X-API-KEY within X-ERROR-MESSAGE will differ if you have configured a custom HTTP header name.*

## Logging

The extension defines a 'restsecurity' logbox logger that, by default, logs to `(yourapp)/logs/restsecurity.log`. Warnings, errors and info will be recorded here.

## Installation

Install the extension to your application via either of the methods detailed below (Git submodule / CommandBox) and then enable the extension by opening up the Preside developer console and entering:

    extension enable preside-ext-rest-security
    reload all

### Git Submodule method

From the root of your application, type the following command:

    git submodule add https://bitbucket.org/hwsdev/preside-ext-rest-security.git application/extensions/preside-ext-rest-security

### CommandBox (box.json) method

From the root of your application, type the following command:

    box install preside-ext-rest-security

## Ideas for the future
* possibility to enable/disable the whole API via the system settings
* possibility to enable/disable the authentication check in the system settings
* possibility to bypass authentication for local development environment
* additional authentication methods
* multiple API keys with labels
* maybe rate-limiting / throttling (general and apikey-specific)